@@ -32,8 +32,15 @@ import base64
 logger = logging.getLogger('beymax.core.client')
 background_tasks = set()
 
+# There's a weird race condition with check_future_dispatch
+# I can't figure out what's causing it, but this certainly resolves it
+future_dispatch_lock = asyncio.Lock() 
+
 # mention_pattern = re.compile(r'<@.*?(\d+)>')
 
+FALLBACK_GENERAL = ''
+
+
 class EventChain(object):
     def __init__(self, eventname):
         self.name = eventname
@@ -69,12 +76,13 @@ class Client(discord.Client):
     This Client is geared towards custom single-server bots, although it can
     serve multiple servers
     """
-    def __init__(self, *args, status=discord.Status.dnd, activity=discord.Game(name="Starting up, please wait..."), **kwargs):
-        super().__init__(*args, status=status, activity=activity, **kwargs)
+    def __init__(self, control_interface, *args, intents=None, status=discord.Status.dnd, activity=discord.Game(name="Starting up, please wait..."), control_meta=None, **kwargs):
+        if intents is None:
+            intents = discord.Intents.default()
+        super().__init__(*args, intents=intents, status=status, activity=activity, **kwargs)
+        self.controller = control_interface
         self.nt = 0
-        self.configuration = {}
-        self.primary_guild = None
-        self.channel_references = {} # reference name -> channel name/id
+        self.channel_references = {} # reference name -> guild id -> channel name/id
         self.event_listeners = {} # event name -> [listener functions (self, event)]
         # changed to set in favor of event API
         self.event_preemption = {} # event name -> counter for preempting beymax-level events
@@ -84,63 +92,62 @@ class Client(discord.Client):
         self.special = [] # list of (check, handler)
         self.suites = {}
         self.channel_lock = asyncio.Lock()
-        config_path = os.environ.get('BEYMAX_CONFIG_PATH', 'config.yml')
-        if os.path.exists(config_path):
-            with open(config_path) as reader:
-                self.configuration = yaml.load(reader, Loader=yaml.SafeLoader)
-        self.command_prefix = self.config_get('prefix', default='!')
         self.tree = discord.app_commands.CommandTree(self)
 
-        self.db = DBConnection(self.config_get('database', 'url'))
-
         # Add the core api tasks and event subscriptions
         APIEssentials.attach(self)
 
+    async def async_init(self):
+        self.db = DBConnection(
+            await self.controller.config_get('database', 'url')
+        )
     
-    async def get_value(self, key, default=None):
+    async def get_value(self, key, guild_id=None, default=None):
         async with self.db.session() as session:
             result = (await session.execute(
                 alchemy.select(APIEssentials.schema.KeyValue).where(
-                    APIEssentials.schema.KeyValue.key == key
+                    (APIEssentials.schema.KeyValue.key == key)
+                    & (APIEssentials.schema.KeyValue.guild_id == guild_id)
                 )
             )).scalars().first()
             if result:
                 return result.value
             return default
 
-    async def set_value(self, key, value):
+    async def set_value(self, key, value, guild_id=None):
         async with self.db.session() as session:
             result = (await session.execute(
                 alchemy.select(APIEssentials.schema.KeyValue).where(
-                    APIEssentials.schema.KeyValue.key == key
+                    (APIEssentials.schema.KeyValue.key == key)
+                    & (APIEssentials.schema.KeyValue.guild_id == guild_id)
                 )
             )).scalars().first()
             if result:
                 result.value = value
             else:
                 session.add_all([
-                    APIEssentials.schema.KeyValue(key=key, value=value)
+                    APIEssentials.schema.KeyValue(key=key, guild_id=guild_id, value=value)
                 ])
             await session.commit()
 
     async def setup_hook(self):
         await self.db.__aenter__()
     
-    def add_context(self, name, nsfw=False, auto_locale_strings=True):
+    def add_context(self, name, nsfw=False, auto_locale_strings=True, _command_suite=None):
         def wrapper(func):
 
             @self.subscribe('before:command-init', once=True)
             async def register_context(self, evt):                
-                
 
-                @self.tree.context_menu(name=name, nsfw=nsfw, guild=self.primary_guild if self.primary_guild is not None else MISSING)
+                @self.tree.context_menu(name=name, nsfw=nsfw)
                 @wraps(func)
                 async def context_wrapper(interaction, *args, **params):
                     logger.info("Invoking context manager {}".format(name))
                     try:
                         return await (interaction.client.dispatch('ctx:{}'.format(name), interaction, *args, **params))
                     except:
-                        logger.error("Exception in context manager", exc_info=True)
+                        traceID = await interaction.client.trace(interaction)
+                        logger.error("Exception in context manager {}. Trace: {}".format(name, traceID), exc_info=True)
                         raise
                     finally:
                         logger.debug("Context complete")
@@ -151,15 +158,30 @@ class Client(discord.Client):
                 
                 @self.subscribe('ctx:{}'.format(name))
                 async def run_context(bot, event, interaction, *args, **params):
-                    print(interaction, args, params)
-                    return await func(interaction, *args, **params)
+                    if _command_suite is None or (bot.controller.suite_enabled_in_guild(_command_suite, interaction.guild) and bot.controller.command_enabled_in_guild(name, interaction.guild)):
+                        try:
+                            return await func(interaction, *args, **params)
+                        except:
+                            traceID = await interaction.client.trace(interaction)
+                            logger.error("Exception in context manager {}. Trace: {}".format(name, traceID), exc_info=True)
+                        finally:
+                            if not interaction.response.is_done():
+                                await interaction.response.send_message(
+                                    "Oops! Something went wrong and I was unable to complete your request",
+                                    ephemeral=True
+                                )
+                    else:
+                        await interaction.response.send_message(
+                            "This command has been disabled by your server's aministrators",
+                            ephemeral=True
+                        )                    
 
             return func
         
         return wrapper
 
 
-    def add_command(self, command, description=None, nsfw=False, parent=None, auto_locale_strings=True, command_suite_group=None): #decorator. Attaches the decorated function to the given command(s)
+    def add_command(self, command, description=None, nsfw=False, parent=None, auto_locale_strings=True, command_suite_group=None, _command_suite=None): #decorator. Attaches the decorated function to the given command(s)
         """
         Decorator. Registers the given function as the handler for the specified command.
         Arguments:
@@ -193,21 +215,39 @@ class Client(discord.Client):
                 callback=func,
                 nsfw=nsfw,
                 parent=parent,
-                guild_ids=[self.primary_guild.id] if self.primary_guild is not None else None,
                 auto_locale_strings=auto_locale_strings,
             )
             # I would like to default to off, but that's apparently impossible to override @everyone
             cmd.default_permissions = discord.Permissions(use_application_commands=True)
-            cmd.subscribe_command(self)
+
+            @self.subscribe('cmd:{}'.format(cmd.name))
+            async def run_cmd(client, event, interaction, **kwargs):
+                if _command_suite is None or (client.controller.suite_enabled_in_guild(_command_suite, interaction.guild) and client.controller.command_enabled_in_guild(command, interaction.guild)):
+                    try:
+                        return await cmd._callback(interaction, **kwargs)
+                    except:
+                        traceID = await interaction.client.trace(interaction)
+                        logger.error("Exception in command {}. Trace: {}".format(cmd.name, traceID), exc_info=True)
+                    finally:
+                        if not interaction.response.is_done():
+                            await interaction.response.send_message(
+                                "Oops! Something went wrong and I was unable to complete your request",
+                                ephemeral=True
+                            )
+                else:
+                    await interaction.response.send_message(
+                        "This command has been disabled by your server's aministrators",
+                        ephemeral=True
+                    ) 
 
             if command_suite_group is None:
+                # If command group is not None, the suite will take care of adding it to the tree
 
                 @self.subscribe('before:command-init', once=True)
                 async def register_command(self, evt):
 
                     self.tree.add_command(
                         cmd,
-                        guild=self.primary_guild if self.primary_guild is not None else MISSING
                     )
             else:
                 command_suite_group.add_command(cmd)
@@ -270,8 +310,13 @@ class Client(discord.Client):
             @self.subscribe(taskname) # This is separated into runtask->task only for triggers/hooks on the task event
             async def task_wrapper(_1, _2):
                 logger.debug("Starting task {}".format(taskname))
-                await func(self)
-                logger.info("Task complete {}".format(taskname))
+                try:
+                    await func(self)
+                except:
+                    traceID = await self.trace()
+                    logger.error("Unhandled exception in task {}. Trace: {}".format(taskname, traceID), exc_info=True)
+                else:
+                    logger.debug("Task complete {}".format(taskname))
             
             self.tasks[taskname]['task'] = run_task
 
@@ -337,7 +382,6 @@ class Client(discord.Client):
                     task_schedule = (await session.execute(
                         alchemy.select(APIEssentials.schema.FutureDispatch).where(
                             (APIEssentials.schema.FutureDispatch.event == taskname)
-                            & (APIEssentials.schema.FutureDispatch.date < time.time())
                         )
                     )).scalars().first()
                     if task_schedule is None:
@@ -352,13 +396,13 @@ class Client(discord.Client):
                 logger.debug("Starting task {}".format(taskname))
                 try:
                     await func(self)
-                    logger.info("Task complete {}".format(taskname))
+                    logger.debug("Task complete {}".format(taskname))
                 finally:
                     await verify_task_schedule(self)
         return wrapper   
             
 
-    def add_special(self, check, flavor='blind'): #decorator. Sets the decorated function to run whenever the check is true
+    def add_special(self, check, flavor='blind', _command_suite=None): #decorator. Sets the decorated function to run whenever the check is true
         """
         Decorator. Sets the decorated function to run whenever the given check function is True.
         Arguments:
@@ -387,7 +431,13 @@ class Client(discord.Client):
             elif flavor == 'modern':
                 def flavor_check(slf, e, msg):
                     return len(msg.content)
-            self.subscribe(event, condition=flavor_check)(lambda s,e,*a,**k: func(s,*a,**k)) # If we add the condition we can double check
+            if _command_suite is None:
+                suite_check = lambda g:True
+            else:
+                suite_check = lambda g: g is None or self.controller.suite_enabled_in_guild(_command_suite, g)
+            def composite_check(client, event, message):
+                return flavor_check(client, event, message) and suite_check(message.guild)
+            self.subscribe(event, condition=composite_check)(lambda s,e,*a,**k: func(s,*a,**k)) # If we add the condition we can double check
             self.special.append((check, event))
             return func
         return wrapper
@@ -411,24 +461,25 @@ class Client(discord.Client):
             event,
             (when - discord.utils.utcnow()).total_seconds()
         ))
-        async with self.db.session() as session:
-            session.add_all([
-                APIEssentials.schema.FutureDispatch(
-                    date=when.timestamp(),
-                    event=event,
-                    pickled_args=base64.b64encode(pickle.dumps(args)).decode(),
-                    pickled_kwargs=base64.b64encode(pickle.dumps(kwargs)).decode(),
+        async with future_dispatch_lock:
+            async with self.db.session() as session:
+                session.add(
+                    APIEssentials.schema.FutureDispatch(
+                        date=when.timestamp(),
+                        event=event,
+                        pickled_args=base64.b64encode(pickle.dumps(args)).decode(),
+                        pickled_kwargs=base64.b64encode(pickle.dumps(kwargs)).decode(),
+                    )
                 )
-            ])
-            await session.commit()
-        self.update_interval(
-            'check_future_dispatch',
-            # Update the next check_future_dispatch invocation to take place ASAP
-            # task runner will trigger in at most 30 seconds
-            # cfd will run and self-update its interval to best match the next dispatch
-            1,
-            permanent=False
-        )
+                await session.commit()
+            self.update_interval(
+                'check_future_dispatch',
+                # Update the next check_future_dispatch invocation to take place ASAP
+                # task runner will trigger in at most 30 seconds
+                # cfd will run and self-update its interval to best match the next dispatch
+                1,
+                permanent=False
+            )
 
     def migration(self, key):
         """
@@ -485,13 +536,16 @@ class Client(discord.Client):
 
             async def handle_event(*args, **kwargs):
                 try:
+                    # NOTE: I considered adding a suite check here, but events are low-enough level to be ambiguous whether or not the event
+                    # will be universally dispatched with a guild context
+                    # Suites should take care to avoid dispatching events from tasks (the only thing that will run in disabled guilds) for guilds which are disabled
                     if condition is None or condition(*args, **kwargs):
                         if once:
                             func.unsubscribe(event)
                         return await func(*args, **kwargs)
                 except:
-                    await self.trace()
-                    logger.error("Unhandled exception in event listener for {}".format(event))
+                    traceID = await self.trace()
+                    logger.error("Unhandled exception in event listener for {}. Trace: {}".format(event, traceID), exc_info=True)
                     raise
 
             handle_event.orig = id(func)
@@ -529,25 +583,29 @@ class Client(discord.Client):
         """
         # creates a channel reference by that name
         # channel references can be changed in configuration
-        self.channel_references[name] = None
+        self.channel_references[name] = {}
 
-    def fetch_channel(self, name):
+    def fetch_channel(self, name, guild_id):
         """
         Fetch the channel object for a given reference name. If the reference is
         undefined, it returns general
         Arguments:
         name : A string channel reference to lookup
         """
-        channel = self.channel_references[name] if name in self.channel_references else None
+        channel = None
+        if name in self.channel_references and guild_id in self.channel_references[name]:
+            channel = self.channel_references[name][guild_id]
         if channel is None:
-            return self._general
+            return self.channel_references[FALLBACK_GENERAL][guild_id]
         return channel
 
-    def enableSuites(self, *suites):
+    def attachSuites(self, *suites):
         """
-        Enables all the given command suites
+        Attaches all the given command suites
         Arguments:
         *suites : One or more CommandSuites
+
+        It is the controller's discretion to enable or disable suites
         """
         for suite in suites:
             suite.attach(self)
@@ -615,26 +673,7 @@ class Client(discord.Client):
             for listener in self.event_listeners[event]
         ]
 
-
-    def config_get(self, *keys, default=None):
-        """
-        Retrieve a given key from the configuration. A multiple keys may be given to retrieve a nested value.
-        Arguments:
-        *keys : List of nested keys to read from the configuration
-        default : (Optional) The fallback value to return if the requested key path does not exist or is undefined. Defaults to None
-        """
-        obj = self.configuration
-        for key in keys:
-            try:
-                if key in obj:
-                    obj = obj[key]
-                else:
-                    return default
-            except TypeError:
-                return default
-        return obj
-
-    async def trace(self, send=True, channel=None):
+    async def trace(self, interaction=None):
         """
         Coroutine. Prints a stack trace to the console, and optionally sends it to the registered
         bugs channel
@@ -644,30 +683,63 @@ class Client(discord.Client):
         x,y,z = sys.exc_info()
         if x is None and y is None and z is None:
             msg = traceback.format_stack()
-            traceback.print_stack()
             logger.debug("Manual stack trace", stack_info=True)
         else:
             msg = traceback.format_exc()
-            logger.debug("Exception stack trace", exc_info=True)
+        timestamp = int(time.time())
+        traceID = '{}/{}/{}'.format(
+            self.shard_id,
+            os.urandom(2).hex(),
+            timestamp
+        )
         if isinstance(msg, list):
             msg = ''.join(msg)
-        if send and self.config_get('send_traces'):
-            await (self.fetch_channel('dev') if channel is None else channel).send(
-                '```{}```'.format(msg[-3990:])
+        async with self.db.session() as session:
+            session.add(APIEssentials.schema.ErrorTrace(
+                traceID=traceID,
+                shardID=self.shard_id,
+                timestamp=timestamp,
+                stack_trace=msg
+            ))
+            await session.commit()
+        if interaction is not None and not interaction.response.is_done():
+            await interaction.response.send_message(
+                "I was unable to complete this action because I encountered an unexpected error.\n"
+                "If you continue to encounter this problem, report it to {} and include this error trace ID\n"
+                "{}".format('(someone)', traceID),
+                ephemeral=True
             )
+        return traceID
 
-    async def shutdown(self):
+    async def shutdown(self, signal=None, event=None):
         """
         Coroutine. Use this function for a clean shutdown.
         Dispatches the 'cleanup' event, waits for all tasks to complete, then disconnects
         the bot
         """
-        await self.change_presence(status=discord.Status.offline)
-        tasks = self.dispatch('cleanup')
-        logger.info("Waiting for cleanup tasks to complete")
-        await tasks
-        await self.close()
-        await self.db.__aexit__(None, None, None)
+        logger.info("Client shutdown initiated{}".format(
+            " by signal {}".format(signal)
+            if signal is not None
+            else ""
+        ))
+        try:
+            await self.change_presence(status=discord.Status.offline)
+            for task in self.tasks.values():
+                if 'task' in task: # Legacy task chains do not set a task attribute
+                    task['task'].stop()
+                    task['task'].cancel()
+            tasks = self.dispatch('cleanup')
+            logger.info("Waiting for cleanup tasks to complete")
+            await tasks
+            await self.close()
+            await self.db.__aexit__(None, None, None)
+            logger.info("Client shutdown complete")
+        except:
+            logger.error("Client shutdown encountered error", exc_info=True)
+        finally:
+            if event is not None:
+                # Signal the controller that the client has shut down
+                event.set()
 
     async def send_rich_message(self, destination, *, content=None, author=None, author_url=None, author_icon_url=None, title=None, description=None, colour=None, footer=None, image=None, thumbnail=None, __video=None, url=None, reference=None, mention_author=True):
         """
@@ -719,7 +791,7 @@ class Client(discord.Client):
         Gets a user object given a form of reference. Optionaly provide a subset of guilds to check
         Arguments:
         reference : A string reference which can either be a user's id or a username to identify a user
-        *guilds : A list of guilds to check. By default, this function checks the primary_guild, then all others
+        *guilds : A list of guilds to check. By default, checks all guilds the bot is joined to
 
         Checks guilds for a user based on id first, then username. Returns the first match
         """
@@ -727,8 +799,6 @@ class Client(discord.Client):
             logger.warning("members intent is not enabled. Consider using client.fetch_member() instead")
         if not len(guilds):
             guilds = list(self.guilds)
-            if self.primary_guild is not None:
-                guilds = [self.primary_guild]
         if isinstance(reference, int):
             for guild in guilds:
                 result = guild.get_member(reference)
@@ -766,6 +836,7 @@ class Client(discord.Client):
             return result.id
         raise NameError("Unable to locate member '%s'. Must use a user ID, username, or username#discriminator" % username)
 
+    # FIXME: Move to subscription
     async def on_message(self, message):
         """
         Coroutine. Default handler for incomming messages. Do not override.
@@ -806,32 +877,6 @@ class Client(discord.Client):
                 self.dispatch(event, message)
                 break
 
-    async def on_guild_join(self, guild):
-        """
-        Coroutine. Handler for joining guilds. Do not override.
-        If you wish to add handling for joining guilds use @bot.subscribe('guild_join')
-
-        If a primary guild is defined and this is not the primary guild, leave it.
-        Otherwise, print a warning that a primary guild is not defined
-        """
-        if self.primary_guild is not None and self.primary_guild != guild:
-            try:
-                await (discord.utils.get(
-                        guild.channels,
-                        name='general',
-                        type=discord.ChannelType.text
-                    )).send_message(
-                    "Unfortunately, this instance of {0} is not configured"
-                    " to run on multiple servers. Please contact the owner"
-                    " of this instance, or run your own instance of {0}."
-                    " Goodbye!".format(self.user.name)
-                )
-            except:
-                pass
-            await guild.leave()
-        elif len(self.guilds) > 1:
-            logger.warning("Joining multiple guilds is not currently supported behavior")
-
     def wait_for(self, event, *, check=None, timeout=None):
         """
         Wait for a single instance of an event.
@@ -862,72 +907,61 @@ class Client(discord.Client):
             return waiter()
         return super().wait_for(event, check=check, timeout=timeout)
 
-APIEssentials = CommandSuite('Beymax Core API Essentials')
+APIEssentials = CommandSuite('Beymax Core API Essentials', force_enable=True)
 
 # Future dispatch events are handled as a task
 # But tasks must be added to an existing bot
 @APIEssentials.add_task(30)
 async def check_future_dispatch(self):
-    now = discord.utils.utcnow()
-    async with self.db.session() as session:
-        results = await session.execute(
-            alchemy.select(APIEssentials.schema.FutureDispatch).where(
-                APIEssentials.schema.FutureDispatch.date <= now.timestamp()
-            )
-        )
-        for event in results.scalars():
-            overshoot = int(now.timestamp() - event.date)
-            if overshoot > 1:
-                logger.warning("Future dispatch {} overshot by {}s. The server may be overloaded".format(
-                    event.event,
-                    overshoot
-                ))
-            await session.delete(event)
-            
-            try:
-                self.dispatch(
-                    event.event,
-                    *pickle.loads(base64.b64decode(as_bytes(event.pickled_args))),
-                    **pickle.loads(base64.b64decode(as_bytes(event.pickled_kwargs)))
+    async with future_dispatch_lock:
+        now = discord.utils.utcnow()
+        async with self.db.session() as session:
+            results = await session.execute(
+                alchemy.select(APIEssentials.schema.FutureDispatch).where(
+                    APIEssentials.schema.FutureDispatch.date <= now.timestamp()
                 )
-            except:
-                logger.error("Unable to load event from future dispatch (data type error?). Please check your database", exc_info=True)
-        
-        results = (await session.execute(
-            alchemy.select(APIEssentials.schema.FutureDispatch).where(
-                APIEssentials.schema.FutureDispatch.date > now.timestamp()
             )
-        )).scalars().all()
-        await session.commit() # Remove the event even if dispatch fails
-        if len(results):
-            self.update_interval(
-                'check_future_dispatch',
-                min(
-                    ceil(evt.date - now.timestamp())
-                    for evt in results
-                ),
-                permanent=False
-            )
-        
+            for event in results.scalars():
+                overshoot = int(now.timestamp() - event.date)
+                if overshoot > 1:
+                    logger.warning("Future dispatch {} overshot by {}s. The server may be overloaded".format(
+                        event.event,
+                        overshoot
+                    ))
+                await session.delete(event)
+                
+                try:
+                    self.dispatch(
+                        event.event,
+                        *pickle.loads(base64.b64decode(as_bytes(event.pickled_args))),
+                        **pickle.loads(base64.b64decode(as_bytes(event.pickled_kwargs)))
+                    )
+                except:
+                    logger.error("Unable to load event from future dispatch (data type error?). Please check your database", exc_info=True)
+            
+            results = (await session.execute(
+                alchemy.select(APIEssentials.schema.FutureDispatch).where(
+                    APIEssentials.schema.FutureDispatch.date > now.timestamp()
+                )
+            )).scalars().all()
+            await session.commit() # Remove the event even if dispatch fails
+            if len(results):
+                self.update_interval(
+                    'check_future_dispatch',
+                    min(
+                        ceil(evt.date - now.timestamp())
+                        for evt in results
+                    ),
+                    permanent=False
+                )
+
+@APIEssentials.add_task(seconds=30)
+async def heartbeat(client):
+    await client.controller.heartbeat()
 
 @APIEssentials.subscribe('ready', once=True)
 async def first_ready(self, event):
     try:
-        if 'primary_guild' in self.configuration:
-            self.primary_guild = discord.utils.get(
-                self.guilds,
-                id=self.configuration['primary_guild']
-            )
-            if self.primary_guild is None:
-                logger.critical("Primary guild set but no matching guild was found")
-                sys.exit("Primary guild set, but no matching guild was found")
-            else:
-                logger.info("Validated primary guild: {}".format(self.primary_guild.name))
-        else:
-            logger.warning("No primary guild set in configuration. This is not supported behavior")
-        for guild in list(self.guilds):
-            logger.debug("Connected to guild {} ({})".format(guild.name, guild.id))
-            await self.on_guild_join(guild)
         logger.debug("{} Tasks in schedule".format(len(self.tasks)))
         self._general = discord.utils.get(
             self.get_all_channels(),
@@ -935,42 +969,25 @@ async def first_ready(self, event):
             type=discord.ChannelType.text
         )
         
-        if 'channels' in self.configuration:
-            for name in self.channel_references:
-                if name in self.configuration['channels']:
-                    channel = discord.utils.get(
-                        self.get_all_channels(),
-                        name=self.configuration['channels'][name],
-                        type=discord.ChannelType.text
-                    )
-                    if channel is None:
-                        channel = discord.utils.get(
-                            self.get_all_channels(),
-                            id=self.configuration['channels'][name],
-                            type=discord.ChannelType.text
-                        )
-                    if channel is None:
-                        raise NameError("No channel by name of "+self.configuration['channels'][name])
-                    self.channel_references[name] = channel
-                else:
-                    logger.warning("Channel reference {} not defined".format(name))
-
-        
+        config_channels = await self.controller.config_get('channels', default={}) # Channel name : guild id : channel id
+        self.channel_references[FALLBACK_GENERAL] = {} # Set up fallback general
+        for guild in self.guilds:
+            channels = guild.text_channels
+            general = discord.utils.get(channels, name='general')
+            if general is None:
+                logging.warning("Unable to find general channel in guild {}. Defaulting to first channel present".format(guild.name))
+                general = channels[0]
+            for channel_name, guild_defs in config_channels.items():
+                if channel_name not in self.channel_references:
+                    self.channel_references[channel_name] = {}
+                self.channel_references[channel_name][guild.id] = discord.utils.get(
+                    channels,
+                    id=guild_defs[str(guild.id)]
+                ) if str(guild.id) in guild_defs else None
+                if self.channel_references[channel_name][guild.id] is None:
+                    logger.warning("Channel reference {} not defined or invalid ID for guild {}".format(channel_name, guild.name))
         
         await (self.dispatch('schema', self.db)) # Returns a task
-        await self.db.connection.run_sync(self.db.base.metadata.create_all)
-        await self.db.connection.commit()
-        taskkey = ''.join(sorted(self.tasks))
-        key = await self.get_value('task_key')
-        if key != taskkey:
-            logger.info("Invalidating task time cache")
-            await self.set_value('task_key', taskkey)
-            async with self.db.session() as session:
-                await session.execute(
-                    alchemy.delete(APIEssentials.schema.Task)
-                )
-        else:
-            logger.debug("Not invalidating task cache")
 
         # Add legacy lock
         DATABASE['lock'] = asyncio.Lock()
@@ -990,6 +1007,18 @@ async def first_ready(self, event):
         
         logger.info("Database ready, now syncing commands")
 
+        taskkey = ''.join(sorted(self.tasks))
+        key = await self.get_value('task_key')
+        if key != taskkey:
+            logger.info("Invalidating task time cache")
+            await self.set_value('task_key', taskkey)
+            async with self.db.session() as session:
+                await session.execute(
+                    alchemy.delete(APIEssentials.schema.Task)
+                )
+        else:
+            logger.debug("Not invalidating task cache")
+
         self.dispatch('command-init')
 
     except:
@@ -999,12 +1028,14 @@ async def first_ready(self, event):
 @APIEssentials.subscribe('command-init', once=True)
 async def sync_commands(self, evt):
     
-    commands = self.tree.get_commands(guild=self.primary_guild)
+    commands = self.tree.get_commands()
 
     logger.debug("Syncing {} commands".format(len(commands)))
 
-    await self.tree.sync(guild = self.primary_guild)
+    await self.tree.sync()
     logger.info("Startup complete")
+    for guild in self.guilds:
+        await self.tree.sync(guild=guild) # Clear out guild commands. We're global-only, baby
     await self.change_presence()
 
 @APIEssentials.subscribe('guild_role_create')
@@ -1014,7 +1045,7 @@ async def update_ignore_roles(self, event, *args):
     """
     Updates Beymax's list of roles to ignore
     """
-    references = self.config_get('ignore_role', default=[])
+    references = await self.controller.config_get('ignore_roles', default=[])
     if not isinstance(references, list):
         references = [references]
     self.ignore_roles = {
@@ -1035,7 +1066,7 @@ async def update_ignore_roles(self, event, *args):
             logger.debug("Ignoring user {}".format(member))
             self.ignored_users.add(member.id)
 
-@APIEssentials.subscribe('after:member_update')
+@APIEssentials.subscribe('after:member_update') # Event not delivered 
 async def update_ignore_users(self, event, before, after):
     """
     Updates Beymax's list of users to ignore
@@ -1048,7 +1079,7 @@ async def update_ignore_users(self, event, before, after):
         self.ignored_users.remove(after.id)
 
 @APIEssentials.add_command('parse', description="Parse this string and run special message handlers")
-async def cmd_parse(interaction, message: Arg('message', description="Message to pass to the bot", type=str)):
+async def cmd_parse(interaction, message: Arg(description="Message to pass to the bot", type=str)):
     msg = lambda : None # Dummy namespace object
     msg.content = message
     msg.author = interaction.user
@@ -1057,13 +1088,13 @@ async def cmd_parse(interaction, message: Arg('message', description="Message to
     await interaction.response.send_message("OK", ephemeral=True)
     await interaction.client.on_message(msg)
 
-@APIEssentials.table
+@APIEssentials.global_table
 class Migration(object):
     __tablename__ = 'core_migrations'
     key = alchemy.Column(alchemy.String, primary_key=True)
     date = alchemy.Column(alchemy.DateTime(timezone=True))
 
-@APIEssentials.table
+@APIEssentials.global_table
 class FutureDispatch(object):
     __tablename__ = 'core_future_dispatch'
     date = alchemy.Column(alchemy.Integer, primary_key=True)
@@ -1075,14 +1106,23 @@ class FutureDispatch(object):
 class KeyValue(object):
     __tablename__ = 'core_key_value'
     key = alchemy.Column(alchemy.String, primary_key=True)
+    guild_id = alchemy.Column(alchemy.BigInteger)
     value = alchemy.Column(alchemy.String)
 
-@APIEssentials.table
+@APIEssentials.global_table
 class Task(object):
     __tablename__ = 'core_tasks'
     name = alchemy.Column(alchemy.String, primary_key=True)
     date = alchemy.Column(alchemy.Integer)
 
+@APIEssentials.global_table
+class ErrorTrace(object):
+    __tablename__ = 'core_traces'
+    traceID = alchemy.Column(alchemy.String, primary_key=True)
+    shardID = alchemy.Column(alchemy.Integer)
+    timestamp = alchemy.Column(alchemy.Integer)
+    stack_trace = alchemy.Column(alchemy.Text)
+
 @APIEssentials.migration('core-migrate-to-db')
 async def migrate_db_backend(bot):
     async with DBView('tasks', 'core_future_dispatch', core_future_dispatch=[], tasks={'key': None, 'tasks': {}}) as db:
@@ -1115,20 +1155,20 @@ async def migrate_db_backend(bot):
             session.add_all(tuples)
             await session.commit()
 
-class MultiserverClient(Client):
-    """
-    This is a multi-server version of the standard Beymax client. This class
-    requires more setup than a standard client.
-    * Supports auto-sharding controlled by a master shard which controls load balance
-    * Supports manual-sharding to allow geo-scaling
-    * Handshake between shards uses a public key exchange
-    * Supports database eventual-consistency for server-specific databases to
-        eventually sync between shards
-    * Supports live configuration changes, with contextual config loaded within
-        each command
-    * Supports background tasks being run globally or in the context of each server
-    * Permissions and more generally, commands, not supported in DMs
+@APIEssentials.migration('add-guild-id')
+async def add_guild_id(client):
+    async with client.db.session() as session:
+        try:
+            await session.execute(
+                "ALTER TABLE core_key_value ADD COLUMN guild_id {};".format(
+                    # Translate the alchemy BigInteger type to a dialect native type
+                    alchemy.BigInteger().compile(client.db.engine.dialect)
+                )
+            )
+            await session.commit()
+        except:
+            logger.warning("Unable to complete add-guild-id migration -- assuming correct schema")
 
-    This client is planned but has no release target
-    """
-    pass
+@APIEssentials.subscribe('guild_join')
+async def on_guild_join(client, event, guild):
+    await client.controller.on_guild_join(guild)
